﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CSharpSA_SDKExample
{
    public partial class Form1 : Form
    {
        public mySA_SDK sa = null;
        SA_Type listType = new SA_Type(TypeIdx.ePointGroup);
        public Form1()
        {
            InitializeComponent();
            SetInitialDialogEntries();
            InitializeSA_SDK();
            SetConnectionStatus();       
        }

        private void SetInitialDialogEntries()
        {
            collection_textbox.Text = "A";
            GroupName_textBox.Text = "RefPts";
            Target_textBox.Text = "p1";
            NumPts_textBox1.Text = "20";
        }

        private void ConnectToSA_btn_Click(object sender, EventArgs e)
        {
            if (sa == null)
            {
                if (!InitializeSA_SDK())
                    return;
            }
            sa.ConnectToSA("localhost");
            SetConnectionStatus();
        }

        bool InitializeSA_SDK()
        {
            if (sa == null)
                sa = new mySA_SDK();
            bool ret = false;
            if (sa == null)
            {
                MessageBox.Show("Failed to initialize SA SDK");
            }
            else
            {
                // MessageBox.Show("SA SDK is initialized");
                ret = true;
            }
            return ret;
        }

        void SetConnectionStatus()
        {
            if (IsSARunning())
            {
                SA_Running_Status_btn.BackColor = Color.LightGreen;
                SA_Running_Status_btn.Text = "SA Running";
            }
            else
            {
                SA_Running_Status_btn.BackColor = Color.OrangeRed;
                SA_Running_Status_btn.Text = "SA Not Running";
            }
            if (sa != null && sa.IsSDKConnected())
            {
                ConnectToSA_btn.BackColor = Color.LightGreen;
                ConnectToSA_btn.Text = "Connected";
            }
            else
            {
                ConnectToSA_btn.BackColor = Color.OrangeRed;
                ConnectToSA_btn.Text = "Not Connected";
            }
        }

        public bool IsSARunning()
        {
            foreach (Process p in Process.GetProcesses("."))
            {
                try
                {
                    string processName = p.ProcessName.ToLower();
                    if (processName.Contains("spatia") && !processName.Contains("spatialanalyzersdk"))
                    {
                        for (int i = 0; i < 20; i++)
                        {
                            p.Refresh();
                            if (p.MainWindowTitle.Length > 0)
                            {
                                return true;
                            }
                            else
                            {
                                Thread.Sleep(2000);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message.ToString());
                }
            }
            return false;
        }

        private void DissconnectFromSA()
        {
            sa = null;
            GC.Collect();
            GC.WaitForFullGCComplete();
            SetConnectionStatus();
        }

        private void Close_btn_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void SA_Running_Status_btn_Click(object sender, EventArgs e)
        {
            SetConnectionStatus();
        }

        private void GetListOfPtGrps_btn_Click(object sender, EventArgs e)
        {
            SA_Type sa_PGType = new SA_Type(TypeIdx.ePointGroup);
            GetListOfObjects(sa_PGType);
            listType.SetType(TypeIdx.ePointGroup);
        }

        private void GetListOfPts_btn_Click(object sender, EventArgs e)
        {
            SA_Items_listBox1.Items.Clear();
            SA_Type sa_PGType = new SA_Type(TypeIdx.ePointGroup);
            int numCollections = 0;
            sa.GetNumberOfCollection(ref numCollections);
            string sCollName = collection_textbox.Text;
            listType.SetType(TypeIdx.eAny);  
            for (int nColl = 0; nColl < numCollections; nColl++)
            {
                if (sa.Get_ithCollectionName(nColl, ref sCollName))
                {
                    List<String> List = new List<string>();
                    if (sa.MakeACollectionObjectNameRefList_ByType(sCollName, sa_PGType, ref List))
                    {
                        foreach (string str in List)
                        {
                            SA_CollObjectName ptGrp = new SA_CollObjectName();
                            ptGrp.SetCollObjectNameType(str);
                            SA_PointNameRefList ptList = new SA_PointNameRefList();
                            if (sa.MakeAPointNameRefListFromAGroup(ptGrp, ref ptList))
                            {
                                foreach (SA_TargetName targ in ptList.ptNameRefList)
                                {
                                    SA_Items_listBox1.Items.Add(targ);

                                }
                            }
                        }
                    }
                }
            }
        }

        private void GetListOfFrames_btn_Click(object sender, EventArgs e)
        {
            SA_Type sa_PGType = new SA_Type(TypeIdx.eFrame);
            GetListOfObjects(sa_PGType);
            listType.SetType(TypeIdx.eFrame);
        }

        private void GetListOfObjects(SA_Type st)
        {
            SA_Items_listBox1.Items.Clear();
            int numCollections = 0;
            sa.GetNumberOfCollection(ref numCollections);
            string sCollName = collection_textbox.Text;
            for (int nColl = 0; nColl < numCollections; nColl++)
            {
                if (sa.Get_ithCollectionName(nColl, ref sCollName))
                {
                    List<String> List = new List<string>();
                    if (sa.MakeACollectionObjectNameRefList_ByType(sCollName, st, ref List))
                    {
                        foreach (string str in List)
                        {                           
                            SA_CollObjectName SA_obj = new SA_CollObjectName();
                            SA_obj.SetCollObjectNameType(str);
                            SA_Items_listBox1.Items.Add(SA_obj);                           
                        }
                    }
                }
            }
        }

        private void ConstructPts_Click(object sender, EventArgs e)
        {
            try
            {
                int numPts = Int32.Parse(NumPts_textBox1.Text);
                sa.ConstructArrayOfPoints(collection_textbox.Text, GroupName_textBox.Text, Target_textBox.Text, numPts);
            }
            catch (FormatException)
            {
                Console.WriteLine($"Unable to parse '{NumPts_textBox1.Text}'");
            }
        }

        private void Delete_Obj_btn_Click(object sender, EventArgs e)
        {
            if (listType.typeIdx == TypeIdx.eAny)
            {
                SA_PointNameRefList pnrl = new SA_PointNameRefList();
                foreach (object element in SA_Items_listBox1.SelectedItems)
                {
                    SA_TargetName tn = new SA_TargetName(element.ToString());
                    pnrl.ptNameRefList.Add(tn);
                }
                sa.DeletePoints(pnrl);
            }
            else
            {
                SA_CollObjectName con = new SA_CollObjectName();
                SA_CollObjectNameList conl = new SA_CollObjectNameList();
                foreach (object element in SA_Items_listBox1.SelectedItems)
                {
                    con.SetCollObjectName(element.ToString());
                    con.saType.SetType(listType.typeIdx);
                    conl.collObjNameRefList.Add(con);
                }
                sa.DeleteObjects(conl);
            }

            RemoveSelectedListItems();
        }

        void RemoveSelectedListItems()
        {
            ListBox.SelectedObjectCollection selectedItems = new ListBox.SelectedObjectCollection(SA_Items_listBox1);
            selectedItems = SA_Items_listBox1.SelectedItems;
            if (SA_Items_listBox1.SelectedIndex > -1)
            {
                for (int i = selectedItems.Count - 1; i >= 0; i--)
                    SA_Items_listBox1.Items.Remove(selectedItems[i]);
            }
        }
    }
}
