﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using SpatialAnalyzerSDK;

namespace CSharpSA_SDKExample
{
    public class mySA_SDK : SpatialAnalyzerSDKClass
    {
        bool connected = false;

        public mySA_SDK()
        {
            connected = false;
        }

        public enum MPStatus
        {
            SdkError = -1,
            Undone = 0,
            InProgress = 1,
            DoneSuccess = 2,
            DoneFatalError = 3,
            DoneMinorError = 4,
            CurrentTask = 5
        };

        public MPStatus IsMPRunning()
        {
            int rCode = 0;
            GetMPStepResult(ref rCode);
            return (MPStatus)rCode;
        }

        public bool ConnectToSA(string host)
        {

            int statusCode = 0;
            connected = ConnectEx(host, ref statusCode);  // what values of status code are possible? 
            if (!connected || statusCode != 0)
            {
                Console.WriteLine("Unable to Connect to SA.  Status Code: " + statusCode);
            }
            else
            {
                SetInteractionMode();
            }
            return connected;
        }

        public bool IsSDKConnected()
        {
            return connected;
        }

        public bool ShutDownSA()
        {
            SetStep("Shut Down SA");
            bool ret = ExecuteStep();

            return ret;
        }

        public bool SetInteractionMode()
        {
            SetStep("Set Interaction Mode");
            // Available options:  
            // "Manual", "Automatic", "Silent", C:\Users\Scott\Documents\Visual Studio 2010\Projects\SA_CADVal_App\SA_DPD_TestPlatform\mycs
            SetSAInteractionModeArg("SA Interaction Mode", "Silent");
            // Available options: 
            // "Halt on Failure Only", "Halt on Failure or Partial Success", "Never Halt", 
            SetMPInteractionModeArg("Measurement Plan Interaction Mode", "Never Halt");
            // Available options: 
            // "Block Application Interaction", "Allow Application Interaction", 
            SetMPDialogInteractionModeArg("Measurement Plan Dialog Interaction Mode", "Allow Application Interaction");
            return ExecuteStep();
        }

        public bool ConstructArrayOfPoints(string c, string g, string t, int n)
        {
            int i = 0;
            for (i = 0; i < n; i++)
            {
                ConstructAPointInWorkingCoordinates(c, g, t + '_' + i, 10.0 * i, 20.1 * i, 30.3 * i);
            }
            return true;
        }

        public bool ConstructAPointInWorkingCoordinates(string c, string g, string t, double x, double y, double z)
        {
            SetStep("Construct a Point in Working Coordinates");
            SetPointNameArg("Point Name", c, g, t);
            SetVectorArg("Working Coordinates", x, y, z);
            return ExecuteStep();
        }

        public bool DeleteObjects(SA_CollObjectNameList rl)
        {
            List<string> refList = new List<string>();
            foreach (SA_CollObjectName objname in rl.collObjNameRefList)
                refList.Add(objname.GetCollObjName());
            return DeleteObjects(ref refList);
        }

        private bool DeleteObjects(ref List<String> objsToDelete)
        {
            SetStep("Delete Objects");
            Object vObjList = null;
            MakeRefObjectList(objsToDelete, ref vObjList);
            SetCollectionObjectNameRefListArg("Object Names", ref vObjList);
            bool ret = ExecuteStep();
            return ret;
        }

        public bool DeletePoints(SA_PointNameRefList pnrl)
        {
            SetStep("Delete Points");
            Object vObjList = null;
            MakePtNameObjectList(pnrl, ref vObjList);
            SetPointNameRefListArg("Point Names", ref vObjList);
            bool ret = ExecuteStep();
            return ret;
        }

        private void MakePtNameObjectList(SA_PointNameRefList pnrl, ref Object vObjList)
        {
            Object o = new ArrayList();
            foreach (SA_TargetName tn in pnrl.ptNameRefList)
                ((ArrayList)o).Add(tn.GetCollGrpTargetName());
            vObjList = new VariantWrapper(((ArrayList)o).ToArray());
        }

        private void MakeRefObjectList(List<String> sLst, ref Object vObjList)
        {
            Object o = new ArrayList();
            foreach (String str in sLst)
                ((ArrayList)o).Add(str);
            vObjList = new VariantWrapper(((ArrayList)o).ToArray());
        }

        public bool MakeACollectionObjectNameRefList_ByType(String coll, SA_Type type, ref List<String> strList)
        {
            SetStep("Make a Collection Object Name Ref List - By Type");
            SetStringArg("Collection", coll);
            // Available options: 
            // "Any", "B-Spline", "Circle", "Cloud", "Scan Stripe Cloud", 
            // "Cross Section Cloud", "Cone", "Cylinder", "Datum", "Ellipse", 
            // "Frame", "Frame Set", "Line", "Paraboloid", "Perimeter", 
            // "Plane", "Point Group", "Poly Surface", "Scan Stripe Mesh", "Slot", 
            // "Sphere", "Surface", "Vector Group", 
            SetObjectTypeArg("Object Type", type.GetTypeName());
            bool ret = ExecuteStep();

            if (ret)
            {
                int rCode = 0;
                GetMPStepResult(ref rCode);
                if (rCode == (int)MPStatus.DoneSuccess)
                {
                    GetCollectionObjectRefListFromSA("Resultant Collection Object Name List", ref strList);
                }
            }
            return ret;
        }

        private bool GetCollectionObjectRefListFromSA(string argName, ref List<String> strList)
        {
            Object o = null;
            VariantWrapper str_list = new VariantWrapper(o);
            Object sl = str_list;

            bool ret = GetCollectionGroupNameRefListArg(argName, ref sl);

            if (ret)
            {
                int rCode = 0;
                GetMPStepResult(ref rCode);
                if (rCode == (int)MPStatus.DoneSuccess)
                {
                    Array a_sl = (Array)sl;

                    for (int i = a_sl.GetLowerBound(0); i <= a_sl.GetUpperBound(0); i++)
                    {
                        String s = (String)a_sl.GetValue(i);
                        strList.Add(s);
                    }
                }
            }
            return ret;
        }

        public bool GetNumberOfCollection(ref int nColl)
        {
            SetStep("Get Number of Collections");
            bool ret = ExecuteStep();
            if (ret)
            {
                int rCode = 0;
                GetMPStepResult(ref rCode);
                if (rCode == (int)MPStatus.DoneSuccess)
                {
                    GetIntegerArg("Total Count", ref nColl);
                }
            }
            return ret;
        }

        public bool Get_ithCollectionName(int nColl, ref string sCollName)
        {
            SetStep("Get i-th Collection Name");
            SetIntegerArg("Collection Index", nColl);
            bool ret = ExecuteStep();
            if (ret)
            {
                int rCode = 0;
                GetMPStepResult(ref rCode);
                if (rCode == (int)MPStatus.DoneSuccess)
                {
                    GetCollectionNameArg("Resultant Name", ref sCollName);
                }
            }
            return ret;
        }

        public bool MakeAPointNameRefListFromAGroup(SA_CollObjectName ptGrp, ref SA_PointNameRefList ptNameList)
        {
            SetStep("Make a Point Name Ref List From a Group");
            SetCollectionObjectNameArg("Group Name", ptGrp.GetCollName(), ptGrp.GetObjName());
            bool ret = ExecuteStep();

            if (ret)
            {
                ret = GetPointRefListFromSA("Resultant Point Name List", ref ptNameList.ptNameRefList);
            }
            return ret;
        }

        private bool GetPointRefListFromSA(string argName, ref List<SA_TargetName> ptNameList)
        {
            Object o = null;
            VariantWrapper str_list = new VariantWrapper(o);
            Object sl = str_list;

            bool ret = GetPointNameRefListArg(argName, ref sl);

            if (ret)
            {
                int rCode = 0;
                GetMPStepResult(ref rCode);
                if (rCode == (int)MPStatus.DoneSuccess)
                {
                    Array a_sl = (Array)sl;

                    for (int i = a_sl.GetLowerBound(0); i <= a_sl.GetUpperBound(0); i++)
                    {
                        String s = (String)a_sl.GetValue(i);
                        SA_TargetName tn = new SA_TargetName(s);
                        ptNameList.Add(tn);
                    }
                }
            }
            return ret;
        }
    }
}
